\\ Copyright 2021, 2022, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
default(recover,0);
read("../vecper.gp");
read("../test-helpers.gp");
read("interv.gp");
read("interv-wip.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");

option_verbose = 0;


\\-----------------------------------------------------------------------------
\\ Generic

ThueMorse(n) = hammingweight(n)%2;
OEIS_check_func("A010060","ThueMorse");

\\ A289813 code ternary 1s into binary
A289813(n) = fromdigits(digits(n,3)%2, 2);
OEIS_check_func("A289813",,['offset,0]);

from_binary(n)=fromdigits(digits(n),2);
OEIS_check_func("A028897","from_binary");

\\-----------------------------------------------------------------------------
\\ A094114  a(n) = -Sum_{i=1..n-1} (-1)^i*2^valuation(i,3).
\\   0, 1, 0, 2, 1, 2, 0, 1, 0, 4, 3, 4, 2, 3, 2, 4, 3, 4, 0, 1, 0, 2, 1, 2,
\\                                |
\\                                |
\\  2           *--*  *--*        |
\\              |  |  |  |        |
\\  1     *--*  |  *--*  |  *--*  |
\\        |  |  |        |  |  |  |
\\  0  *--*  *--*        *--*  *--*
\\      0  1  2  3  4  5  6  7  8   9

A094114_term(n) = (-1)^n * 2^valuation(n, 3);
{
  \\ terms absolute values = A318666
OEIS_check_func("A318666",(n)->abs(A094114_term(n)));

  check_equal(vector(3^7,n,   A094114_term(3*n)),
              vector(3^7,n, 2*A094114_term(n)));
  check_equal(vector(3^7,n,   A094114_term(3*n + 1)),
              vector(3^7,n, - A094114_term(3*n + 2)));

  check_equal(vector(3^7,n, A094114_term(3*n + 1)),
              vector(3^7,n, - (-1)^n));
  check_equal(vector(3^7,n, A094114_term(3*n + 2)),
              vector(3^7,n, (-1)^n));
}

A094114_by_sum(n) =
{
  n>=1 || error("A094114 is for n>=1");
  - sum(i=1, n-1, A094114_term(i));
}
OEIS_check_func("A094114_by_sum",,['offset,1]);

\\ +/-1 alternating replaces each ternary 1
\\ a(n) = Sum_{i=0..k} (d[i] mod 2)*(-1)^(d[k]+..+d[i+1]) * 2^i, where n-1 = ternary digits d[k]..d[0].
\\
A094114(n) =
{
  n>=1 || error("A094114 is for n>=1");
  n--; my(v=digits(n,3),s=-1); for(i=1,#v, v[i]=if(v[i]%2, s=-s)); fromdigits(v,2);
}
{
  my(v=vector(3^9,i,A094114_term(i)));
  for(i=2,#v, v[i]+=v[i-1]);
  v=concat(0,-v);
  check_equal(v[1..100],
              vector(100,n,A094114_by_sum(n)),
              "A094114 sum of terms");
  check_equal(vector(#v,n,A094114(n)),
              v);

  check_equal(vector(3^7,n, A094114(3*n)),
              vector(3^7,n, 2*A094114(n)),
              "A094114(3*n)");
  check_equal(vector(3^7,n, A094114(3*n-1)),
              vector(3^7,n, 2*A094114(n) - (-1)^n),
              "A094114(3*n-1)");
  check_equal(vector(3^7,n, A094114(3*n-2)),
              vector(3^7,n, 2*A094114(n)),
              "A094114(3*n-2)");
}


\\-----------------------------------------------------------------------------

\\ A065620 reversing binary alternating sum
\\ 1-bits alternately +1,-1 starting at the lowest as =1
\\ Knuth "reversing binary" volume 2 section 4.1 exercise 27 (pdf 239)
A065620(n) =
{
  n >= 0 || error();
  my(v=binary(n),s=-1);
  forstep(i=#v,1,-1, if(v[i], v[i]=(s=-s)));
  fromdigits(v,2);
}
OEIS_check_func("A065620");

\\ A104895 reversing binary alternating sum, opposite signs
\\ lowest 1-bit is -1
A104895(n) = - A065620(n);
OEIS_check_func("A104895");

\\ bits alternating +/-1 with +1 highest
alternating_bits(n) = \
  my(v=binary(n),s=-1); for(i=1,#v, if(v[i], v[i]=(s=-s))); fromdigits(v,2);
{
  check_equal(alternating_bits(3), 2 - 1);
  check_equal(alternating_bits(7), 4 - 2 + 1);
  \\ vector(50,n, alternating_bits(n))
  \\ not in OEIS: 1, 2, 1, 4, 3, 2, 3, 8, 7, 6, 7, 4, 5, 6, 5, 16, 15, 14, 15,12

  \\ hammingweight to make lowest 1-bit +1
  check_equal(vector(#v,n,n--; A065620(n)),
              vector(#v,n,n--; alternating_bits(n) * -(-1)^hammingweight(n)),
              "A065620 vs alternating_bits");
}

\\ A063694 odd-position bits cleared to 0
\\ same increments as alternating_bits()
A063694(n) = bitand(n, if(n, 2<<bitor(1,logint(n,2))\3));
{
  OEIS_check_func("A063694");
  check_equal(vector(1024,n, abs(alternating_bits(n+1) - alternating_bits(n))),
              vector(1024,n, abs(A063694(n+1) - A063694(n))),
              "alternating_bits same abs diffs as A063694 clear odd position bits");
}
\\ cf A063695 even position bits cleared to 0

{
  check_equal(vector(3^8,n, A094114(n)),
              vector(3^8,n, abs(A065620(A289813(n-1)))),
              "A094114 by reversing binary A065620");
  check_equal(vector(3^8,n, A094114(n)),
              vector(3^8,n, abs(A104895(A289813(n-1)))),
              "A094114 by negated reversing binary A104895");

  check_equal(vector(3^8,n, A094114(n)),
              vector(3^8,n, alternating_bits(A289813(n-1))),
              "A094114 by alternating_bits");
}

A292600(n) = alternating_bits(n) - (hammingweight(n)%2);
{
  OEIS_check_func("A292600");

  check_equal(A292600(from_binary(1000100001010)),
                      from_binary( 111100000110));

  check_equal(A292600(from_binary(1000100001000)),
                      from_binary( 111100000111));
}

\\-----------------------------------------------------------------------------
\\ A332497
\\   0, 1, 0, 3, 2, 3, 0, 1, 0, 7, 6, 7, 4, 5, 4, 7, 6, 7, 0, 1, 0, 3, 2, 3,
\\   0, 1, 0, 2, 1, 2, 0, 1, 0, 4, 3, 4, 2, 3, 2, 4, 3, 4, 0, 1, 0, 2, 1, 2,
\\
\\                                | to 7
\\  3           *--*  *--*        |
\\              |  |  |  |        |
\\  2           |  *--*  |        |
\\              |        |        |
\\  1     *--*  |        |  *--*  |
\\        |  |  |        |  |  |  |
\\  0  *--*  *--*        *--*  *--*
\\      0  1  2  3  4  5  6  7  8   9


\\ A006068
UnGray(g) = A006068(g);
A006068(g) =
{
  \\ but cf Joerg by halvings
  my(v=binary(g));
  for(i=2,#v,    \\ high to low, so each bit propagates down
      v[i] = bitxor(v[i],v[i-1]));
  fromdigits(v,2);
}
{
  OEIS_check_func("A006068");
  check_equal(vector(2^8,n,n--;
                     my(v=binary(n),s=0);
                     for(i=1,#v, v[i]=if(v[i]%2, s=1-s, s));
                     fromdigits(v,2)),
              vector(2^8,n,n--; UnGray(n)),
              "UnGray by bit flips");
}

\\ Remy Sigrist in A332497
A332497(n) = my(x=0,k=1); while(n, if(n%3==1, x=2^k-1-x); n\=3; k++); x;
{
  OEIS_check_func("A332497");

  check_equal(vector(3^8,n,n--; A332497(n)),
              vector(3^8,n,n--;
                     my(v=digits(n,3)%2,s=0);
                     for(i=1,#v, v[i]=if(v[i]%2, s=1-s, s));
                     fromdigits(v,2)),
              "A332497 by bit flips");

  check_equal(vector(3^8,n,n--; A332497(n)),
              vector(3^8,n,n--; A006068(A289813(n))),
              "A332497 by UnGray");

  check_equal(vector(3^8,n,n--; A332497(n)),
              vector(3^8,n,n--; 2*A094114(n+1) - (n%2)),
              "A332497 as 2*A094114");


  check_equal(vector(3^8,n, A094114(n)),
              vector(3^8,n, ceil(A332497(n-1)/2)),
              "A094114 as A332497/2");
}
{
  if(0,
     my(func=A094114, offset=1);
     my(func=A332497, offset=0);
     my(k=9,
        hi=3^k,
        xv=List([]),
        yv=List([]));
     listput(xv,0);
     listput(yv,2^k);
     for(x=0,hi-1,
        my(y=func(x+offset));
        listput(xv,x);
        listput(yv,y);
        listput(xv,x+1);
        listput(yv,y));
     listput(xv,hi);
     listput(yv,2^k);

     xv=Vec(xv);
     yv=Vec(yv);
     plothraw(xv,yv,1);
     quit);
}


\\-----------------------------------------------------------------------------

logbase(b,x)=log(x)/log(b)
log2(x)=logbase(2,x)
log3(x)=logbase(3,x)
log_2=log2;
log_3=log3;

i_validate(i) = (0 <= i && i < n) || error();
j_validate(j) = (0 <= j && j < m) || error();
R_validate() =
{
  n >= m || error("must n>=m");
  for(p=1,#R,
     my(i,j); [i,j]=R[p];
     i_validate(i);
     j_validate(j));
  #Set(R) == #R || error("R elements not distinct");
}

\\ num elements in R
r() = #R;

have_ij(i,j) =
{
  i_validate(i);
  j_validate(j);
  for(p=1,r(),
     if(R[p][1]==i && R[p][2]==j,
        return(1)));
  0;
}
R_print() =
{
  forstep(j=m-1,0,-1,
     for(i=0,n-1,
        print1("  ",if(have_ij(i,j),"X",".")));
     print());
}

column_is_not_empty(i) =
{
  i_validate(i);
  for(j=0,m-1, if(have_ij(i,j), return(1))); 0;
}
row_is_not_empty(j) =
{
  j_validate(j);
  for(i=0,n-1, if(have_ij(i,j), return(1))); 0;
}

Hironaka_Hausdorff_dimension = log_2(1 + 2^log_3(2));
print("Hironaka_Hausdorff_dimension = ",Hironaka_Hausdorff_dimension);
\\ not in OEIS: 1.34968382019557
\\ not in OEIS: 6,8,3,8,2,0,1,9,5,5,7
check_equal(nearly_equal(Hironaka_Hausdorff_dimension, log_2(1 + 2^(log(2)/log(3)))), 1);

Hironaka_metric_dimension = 1 + logbase(3,3/2);
print("Hironaka_metric_dimension = ",Hironaka_metric_dimension);
\\ not in OEIS: 1.36907024642
\\ not in OEIS: 9,0,7,0,2,4,6,4,2

check_equal(nearly_equal(Hironaka_metric_dimension, 2-log_3(2)), 1);

\\ log_3(2)
\\ A102525 Decimal expansion of log(2)/log(3).
\\ 1 + 2^log_3(2)
\\ not in OEIS: 2.5485626526302


\\-----------------------------------------------------------------------------
print("Hironaka pattern");
\\         +---+---+---+
\\     j=1 |   | S |   |        t(1) = 1 in row
\\         +---+---+---+
\\     j=0 | S |   | S |        t(0) = 2 in row
\\         +---+---+---+
\\         i=0  i=1  i=2
\\
n = 3;
m = 2;
R = [ [0,0], [1,1], [2,0] ]
R_print();
R_validate();

\\ number of i with (i,j) in R       (page 1 and end of page 8)
t(j) =
{
  j_validate(j);
  sum(i=0,n-1, have_ij(i,j));
}

dimR() =
{
  n >= m || error("must n>=m");
  logbase(m, sum(j=0,m-1, t(j)^logbase(n,m)))
}
dimR()

\\ number of j where (i,j) in R for some i, how many not-empty rows
s() = sum(j=0,m-1, row_is_not_empty(j));
check_equal(r(), 3, "r");
check_equal(s(), 2, "s");

mdimR() = logbase(m,s()) + logbase(n, r()/s());
mdimR()

{
  check_equal(logbase(n,m), log3(2));
  check_equal(t(0), 2);
  check_equal(t(1), 1);
  check_equal(nearly_equal(t(0)^logbase(n,m), 2^log3(2)),
              1, "t(1) power");
  check_equal(nearly_equal(t(1)^logbase(n,m), 1.0),
              1, "t(0) power");
  check_equal(vector(2,j,j--; t(j)), [2,1]);
  check_equal(nearly_equal(dimR(), Hironaka_Hausdorff_dimension),
              1, "dimR()");
}
log2(1+2^log3(2))

{
  localprec(1000);
  my(d=dimR());
  check_equal(precision(d),1001);
  my(v=digits(floor(d*10^104)));
  for(i=1,#v,print1(v[i],",");if(i==35||i==70,print()))
}

print();

\\-----------------------------------------------------------------------------
print("Triangle Pattern");
\\ X  .  .
\\ X  X  .
\\ pattern doesn't matter, only one row 1 and other row 2

R = [ [0,0], [0,1], [1,0] ]
R_print();
R_validate();
{
  check_equal(r(), 3);
  check_equal(t(1), 1);
  check_equal(t(0), 2);
  check_equal(nearly_equal(dimR(), Hironaka_Hausdorff_dimension),
              1, "dimR() = Hironaka");
}
dimR()
mdimR()

print();

\\-----------------------------------------------------------------------------
print("One Row Only");

{
  R = [ [0,1], [1,1], [2,1] ];
  check_equal(nearly_equal(dimR(), 1.0), 1,
              "R is a line");
  R = [ [0,0], [1,0], [2,0] ];
  check_equal(nearly_equal(dimR(), 1.0), 1,
              "R is a line");
}

R = [ [0,0], [1,0], [2,0] ]
R_print();
R_validate();
{
  check_equal(r(), 3);
  check_equal(t(1), 0);
  check_equal(t(0), 3);
  check_equal(nearly_equal(t(1)^logbase(n,m), 0.0),
              1, "t(1) power");
  check_equal(nearly_equal(t(0)^logbase(n,m), 2.0),
              1, "t(0) power");
  check_equal(vector(2,j,j--; t(j)), [3,0]);
  check_equal(nearly_equal(dimR(), 1.0),
              1, "dimR() = 1");
  check_equal(nearly_equal(mdimR(), 1.0),
              1, "mdimR() = 1");
}
print();

\\-----------------------------------------------------------------------------
print("Full Grid = 2 Dimensional");

R = {[ [0,1], [1,1], [2,1],
       [0,0], [1,0], [2,0] ]};
R_print();
R_validate();
{
  check_equal(r(), 6);
  check_equal(t(1), 3);
  check_equal(t(0), 3);
  check_equal(nearly_equal(t(1)^logbase(n,m), 2),
              1, "t(1) power");
  check_equal(nearly_equal(t(0)^logbase(n,m), 2),
              1, "t(0) power");
  check_equal(vector(2,j,j--; t(j)), [3,3]);
  check_equal(nearly_equal(dimR(), 2),
              1, "dimR() = 2");
  check_equal(nearly_equal(mdimR(), 2),
              1, "mdimR() = 2");
}

{
  forstep(len=6,1,-1,
     R=R[1..len];
     #R==len || error();
     print("len "len"  mdimR = ",mdimR()));
}
print();

\\-----------------------------------------------------------------------------
print("Same Num Each Row");
R = [ [0,0], [2,1] ]
R_print();
R_validate();
t(1)^logbase(n,m)
{
  check_equal(r(), 2);
  check_equal(t(1), 1);
  check_equal(t(0), 1);
  check_equal(nearly_equal(t(1)^logbase(n,m), 1.0),
              1, "t(1) power");
  check_equal(nearly_equal(t(0)^logbase(n,m), 1.0),
              1, "t(0) power");
  check_equal(vector(2,j,j--; t(j)), [1,1]);
  check_equal(nearly_equal(dimR(), 1.0),
              1, "dimR() = 1");
  check_equal(nearly_equal(mdimR(), 1.0),
              1, "mdimR() = 1");
}

print();

\\-----------------------------------------------------------------------------
print("Bedford's Examples");

m = 4; n = 5;
R = [ [0,0], [4,0], [0,3], [4,3] ]
R_print();
R_validate();
\\ logbase(5,4) 0.861353116146 A153101
{
  check_equal(r(), 4);
  check_equal(vector(4,j,j--; t(j)), [2,0,0,2], "t vector");
  check_equal(nearly_equal(mdimR(), logbase(m,2) + logbase(n,2)),
              1, "mdimR()");
  check_equal(nearly_equal(dimR(), mdimR()),
              1, "dimR() = mdimR()");
}

m = 2; n = 4;
R = [ [0,0], [1,1], [2,1], [3,1] ]
R_print();
R_validate();
\\ log_2(1+sqrt(3))
\\ log(1+sqrt(3))/log(2)
\\ dimR()
\\ not in OEIS: 1.4499843134764
\\ log(1+sqrt(3))
\\ not in OEIS: 1.0050525387423810090
\\ 1+sqrt(3) = A090388
{
  check_equal(r(), 4);
  check_equal(vector(2,j,j--; t(j)), [1,3], "t vector");
  check_equal(nearly_equal(mdimR(), 3/2),
              1, "4x2 3,1 mdimR()");
  check_equal(nearly_equal(logbase(4,2), 1/2), 1);
  check_equal(nearly_equal(dimR(), log_2(1+sqrt(3))),
              1, "4x2 3,1 dimR() = log_2(1+sqrt(3)");
}
\\ vector(32,n, fromdigits(apply(d->d!=0,digits(n,4)),2))
\\ not in OEIS: 1,1,1,2,3,3,3,2,3,3,3,2,3,3,3,4,5,5,5,6,7,7,7,6,7,7,7,6,7,7,7,4
\\ my(x=[1..4^5], y=[fromdigits(apply(d->d!=0,digits(n,4)),2) | n<-x]); \
\\   plothraw(x,y,1)


\\--------
\\ with 2,2 rows both Hausdorff and metric are 3/2
m = 2; n = 4;
R = [ [0,0], [1,0], [2,1], [3,1] ]
R_print();
R_validate();
{
  check_equal(r(), 4);
  check_equal(vector(2,j,j--; t(j)), [2,2], "4x2 2,2 t vector");
  check_equal(nearly_equal(mdimR(), 3/2),
              1, "4x2 2,2 mdimR() same as 3,1");
  check_equal(nearly_equal(dimR(), mdimR()),
              1, "4x2 2,2 dimR() = mdimR()");
}

\\-----------------------------------------------------------------------------
\\ Bedford
\\
\\ r > s > 1   width r height s
\\ grid lines i=0..s vertical   j=0..r horizontal
\\ rows i = 0..s-1
\\ k_i = num blocks in row i
\\ t = num not-empty rows (k_i != 0)
\\
\\           /   1       1   \              log( sum k_i )
\\ cap E  =  | ----- - ----- | * log(t)  +  --------------
\\           \ log s   log r /                  log r
\\
\\ r=3;s=2;t=2;
\\ (1/log(m) - 1/log(n))*log(r) + log(r)/log(n)
\\ log(r)/log(m) - log(r)/log(n)  + log(r)/log(n)
\\ log(s)/log(m) + log(r/s)/log(n)
\\ log(s)/log(m) + log(r)/log(n) - log(s)/log(n)
\\
\\ pbar = p_0, ..., p_{s-1}
\\ (sum p_i) = 1
\\ p_i = 0 if k_i = 0
\\ H(pbar) = sum p_i * log(p_i)
\\ y = 0.y1 y2 ... base s
\\ P(y,n,i) = count y_j = i for 1<=j<=n
\\                             / - sum p_i*log(p_i)        p_i * log(k_i) \
\\ Theorem 4.5  dim E = sup    | ------------------- + sum -------------- |
\\                       pbar  \         log s                 log r      /
\\ (pbar,p) by
\\ 0 = -p + sum p_i*log(k_i) / log(r)

{
  my(r=3, s=2,
     t=2,         \\ rows not empty
     total = 3);
  check_equal(nearly_equal( (1/log(s) - 1/log(r)) * log(t) + log(total)/log(r),
                          Hironaka_metric_dimension),
             1,
              "Bedford Hironaka_metric_dimension");
}


\\ p = 1/2 + (1 - sqrt(3)/2)
\\ q = 1/2 - (1 - sqrt(3)/2)
\\ log( (-p*log(p) -q*log(q))/log(2) + (p*log(3) + q*log(1))/log(4) )
\\ log( (-p*log(p) -q*log(q)) / log(s) + (log(3)*p + log(1)*q)/log(r) )
\\ log(1+sqrt(3))/log(2) - dd
\\ p + q = 1
\\ log( (-p*log(p) -(1-p)*log(1-p)) / log(s) + log(3)*p /log(r) )
\\ log( -p*log(p)/log(2) - (1-p)*log(1-p)/log(2) + p*log(3)/log(4) )
\\
\\ ploth(p=.001,.999, my(q=1-p); \
\\    (-p*log(p) -q*log(q))/log(2) + (p*log(3) + q*log(1))/log(4) )
\\ ploth(p=.001,.999, \
\\    p*log(3)/2/log(2) - p*log(p)/log(2) - (1-p)*log(1-p)/log(2))

\\ ploth(p=.001,.999, p*log(3)/2 - p*log(p) - (1-p)*log(1-p))
\\ ploth(p=.001,.999, log((sqrt(3)/p)^p) - log((1-p)^(1-p)) )
\\ ploth(p=.001,.999, log( (sqrt(3)/p)^p / (1-p)^(1-p) )  )
\\ ploth(p=.001,.999, (sqrt(3)/p)^p / (1-p)^(1-p)  )

\\ my(p=s); dd = p*log(3)/log(4) - p*log(p)/log(2) - (1-p)*log(1-p)/log(2)
\\ default(realprecision,200);
\\ default(seriesprecision,100);
\\ my(p=x+1/2, g= p*log(3)/log(4) - p*log(p)/log(2) - (1-p)*log(1-p)/log(2)); \
\\ g=Pol(g'); s=solve(x=-1/2,1/2, subst(g,'x,x)); s+=1/2

\\ A334843  (3 - sqrt(3))/2
\\ A152422  (sqrt(3) - 1)/2


\\-----------------------------------------------------------------------------
\\ McMullen
\\
\\ X metric space.
\\ Sets X[i] are eps-cover if X = union X[i], and all diam(X[i]) < eps
\\
\\ delta-dimensional Hausdorff measure of X
\\
\\ mu_delta(X) = sup inf( sum diam(X[i])^delta )   where X[i] an eps-cover
\\          eps>0
\\ dim X = inf delta where mu_delta(X) = 0
\\       = sup delta where mu_delta(X) = infinity

\\ psi maps digits i_1, i_2, ... -> Sum x_i_k / n^k, Sum y_i_k / n^k
\\ coordinates
\\ surjection because some multiple representations .1000.. vs .0999..

\\ metric dimension is coverings all elements same size
\\     metric dimension >= Hausdorff dimension
\\
\\ same when n=m square,
\\ or those t(j) != 0 are all the same value    same num in each row
\\ mdimR = logbase(m,s) + logbase(n, r/s);
\\                  notempty                  total/notempty
\\
\\ dimR = logbase(m, sum(j=0,m-1, t(j)^logbase(n,m)))

print();

\\-----------------------------------------------------------------------------
print("4x2 case log_2(1+sqrt(3))");

range_bits(t) = t=abs(t[1]-t[2]); logint(denominator(t),2) - logint(numerator(t),2);
check_equal(range_bits([9,9+1/2^16]), 16);

calculate_digits(scale) =
{
  print("digits scale ",scale);
  t_log2 = interv_log([2,2], scale);
  print("  t_log2 = "t_log2*1.0," bits ",range_bits(t_log2));

  t_sqrt3 = interv_solve_bisection([0,3],x->x^2-3, scale);
  print("  t_sqrt3 = "t_sqrt3*1.0," bits ",range_bits(t_sqrt3));

  t_sqrt3_plus_1 = interv_add_constant(t_sqrt3,1);
  print("  t_sqrt3_plus_1 = "t_sqrt3_plus_1*1.0," bits ",range_bits(t_sqrt3_plus_1));

  t_log_sqrt3plus1 = interv_log(t_sqrt3_plus_1, scale);
  print("  t_log_sqrt3plus1 = "t_log_sqrt3plus1*1.0," bits ",range_bits(t_log_sqrt3plus1));

  t_dimR = interv_div(t_log_sqrt3plus1,t_log2);
  print("  t_dimR="t_dimR*1.0," bits ",range_bits(t_dimR));
}


calculate_digits(600);
{
  my(want=OEIS_data("A343620"),
     v=interv_frac_digits(t_dimR,,105));
  print("v length ",#v);
  for(i=1,105, print1(v[i],","); if(i%35==0,print()));
  check_equal(v,want, "interval vs A343620 samples");

  localprec(10000);
  my(H=log_2(1+sqrt(3)),
     u=digits(floor(H*10^104)));
  check_equal(u,v,
              Str("interval vs floor of prec ",precision(H)));
}

print();

\\-----------------------------------------------------------------------------
\\ Dimension Digits
\\ dimR
\\ log(1 + 2^(log(2)/log(3))) / log(2)
\\ log(1 + exp(log(2)^2 / log(3))) / log(2)
\\
\\ mdimR   2-log_3(2)

calculate_digits(scale) =
{
  print("digits scale ",scale);
  t_log2 = interv_log([2,2],10*scale);
  print("  t_log2 = "t_log2*1.0," bits ",range_bits(t_log2));

  t_log3 = interv_log([3,3],20*scale);
  print("  t_log3 = "t_log3*1.0," bits ",range_bits(t_log3));

  t_logbase_3_of_2 = interv_div(t_log2,t_log3);
  print("  t_logbase_3_of_2 = "t_logbase_3_of_2*1.0," bits ",range_bits(t_logbase_3_of_2));

  \\ 2 - log(2)/log(3)
  t_mdimR = interv_add_constant(interv_neg(t_logbase_3_of_2), 2);
  print("  t_mdimR = ",t_mdimR*1.0," bits ",range_bits(t_mdimR));

  \\ log(2)^2/log(3)
  \\ not in OEIS: 0.43732717981943680
  my(E = interv_div(interv_pow_constant(t_log2,2), t_log3));
  E = interv_widen_bits(E,20*scale);
  print("  E = log(2)^2/log(3) =",E*1.0," bits ",range_bits(E));

  \\ 1 + 2^(log(2)/log(3))
  \\ not in OEIS: 2.5485626526302429
  my(F = interv_add_constant(interv_exp(E,scale), 1));
  F = interv_widen_bits(F,5*scale);
  print("  F = 1 + 2^logbase_3_of_2 = ",F*1.0," bits ",range_bits(F));

  t_logF = interv_log(F, 8*scale);
  print("  log(1+F) = "t_logF*1.0," bits ",range_bits(t_logF));

  t_dimR = interv_div(t_logF,t_log2);
  print("  t_dimR="t_dimR*1.0," bits ",range_bits(t_dimR));
}

calculate_digits(90);
{
  \\ A102525 Decimal expansion of log(2)/log(3).
  my(v=OEIS_data("A102525"));
  v=concat(0,v);
  check_equal(interv_frac_digits(t_logbase_3_of_2,,#v),
              v,
              "A102525 log_3(2)");
}

{
  my(want=OEIS_data("A346639"),
     v=interv_frac_digits(t_dimR,,#want));
  print("dimR "v);
  print("v length ",#v,"  with available ",#interv_frac_digits(t_dimR));
  check_equal(v,want, "interval vs A346639 samples");

  localprec(2000);
  my(u=digits(floor(log2(1+ 2^log3(2)) *10^104)));
  check_equal(v,u, "interval vs floor");
  for(i=1,105, print1(v[i],","); if(i%35==0,print()));
}
{
  print();
  my(want=OEIS_data("A346640"),
     v=interv_frac_digits(t_mdimR,,#want));
  print("mdimR "v);
  print("v length ",#v,"  with available ",#interv_frac_digits(t_dimR));
  check_equal(v,want, "interval vs A346640 samples");

  \\ DATA print
  localprec(2000);
  my(u=digits(floor( (2-log3(2)) *10^104)));
  check_equal(u,v, "interval vs floor");
  for(i=1,105, print1(v[i],","); if(i%35==0,print()));
}

\\ 5000*log2(10)  \\ 16609
\\ 2^8
\\ exp(log(2) * 8)

\\-----------------------------------------------------------------------------
print("end");
